"""
This file is a part of Internshala-bot Package. 
Github - https://github.com/Eviltr0N/internshala-bot
Written by - Mayank Lodhi
"""


from undetected_playwright.sync_api import sync_playwright
import json
import os
import time
from .resume_handler import Skills, Certifications, Projects, cover_letter_prompt_format, assignment_prompt_format, assignment_validation_prompt



class chat:
    def __init__(self, browser_inst):
        self.main_page_url = "https://chatgpt.com/?model=auto"
        self.cover_letter_url = None
        self.assignment_url = None
        self.gpt_check_asg_url = None
        config_dir = os.path.join(os.getcwd(), '.config')
        self.gpt_state_conf = os.path.join(config_dir, 'chat_gpt_state.json')
        self.page = browser_inst.new_page()

    def get_cover_letter(self, profile, company, about, skills):
        prompt = cover_letter_prompt_format(profile, company, about, skills)
        if self.cover_letter_url is None:
            self.page.goto(self.main_page_url , timeout=60000, wait_until='networkidle')
        else:
            self.page.goto(self.cover_letter_url , timeout=60000, wait_until='networkidle')
        time.sleep(2)
        self.page.locator('textarea#prompt-textarea').fill(prompt)
        time.sleep(2)
        self.page.locator('[data-testid="send-button"]').click()
        print('Generating Cover Letter...')
        time.sleep(10)
        while self.page.locator('[data-testid="stop-button"]').is_visible():
            time.sleep(2)
        resp = self.page.locator('div.text-base').all()[-3].inner_text()
        resp = resp.replace("ChatGPT\nMemory updated\n\n", "").replace("ChatGPT\n\n", "").replace("\n\n4o mini", "").replace("\n\n4o","").replace("\nIs this conversation helpful so far?", "").replace('Which response do you prefer?\nYour choice will help make ChatGPT better.\nChatGPT\nResponse 1', "").replace('ChatGPT\nResponse 2', '')
        if len(resp) > 2048:
            cover_lt=''
            for x in resp.split("."):
                if len(cover_lt) + len(x) + 1 <= 2000:
                    cover_lt += x + "."
                else:
                    break
        else:
            cover_lt = resp
        
        self.cover_letter_url = self.page.url
        self.page.context.storage_state(path=self.gpt_state_conf)
        return cover_lt


    def get_assignment_answer(self, profile, company, about, skills, question):
        prompt = assignment_prompt_format(profile, company, about, skills, question)
        if self.assignment_url is None:
            self.page.goto(self.main_page_url , timeout=60000, wait_until='networkidle')
        else:
            self.page.goto(self.assignment_url , timeout=60000, wait_until='networkidle')
        time.sleep(2)
        self.page.locator('textarea#prompt-textarea').fill(prompt)
        time.sleep(1)
        self.page.locator('[data-testid="send-button"]').click()
        time.sleep(5)
        print('Solving assignment...')
        while self.page.locator('[data-testid="stop-button"]').is_visible():
            time.sleep(2)
        resp = self.page.locator('div.text-base').all()[-3].inner_text()
        resp = resp.replace("ChatGPT\nMemory updated\n\n", "").replace("ChatGPT\n\n", "").replace("\n\n4o mini", "").replace("\n\n4o","").replace("\nIs this conversation helpful so far?", "").replace('Which response do you prefer?\nYour choice will help make ChatGPT better.\nChatGPT\nResponse 1', "").replace('ChatGPT\nResponse 2', '')
        self.assignment_url = self.page.url
        self.page.context.storage_state(path=self.gpt_state_conf)
        return resp

    def assmnt_is_valid(self, profile, question):
        prompt = assignment_validation_prompt(profile, question)
        if self.gpt_check_asg_url is None:
            self.page.goto(self.main_page_url , timeout=60000, wait_until='networkidle')
        else:
            self.page.goto(self.gpt_check_asg_url , timeout=60000, wait_until='networkidle')
        time.sleep(2)
        self.page.locator('textarea#prompt-textarea').fill(prompt)
        time.sleep(1)
        self.page.locator('[data-testid="send-button"]').click()
        print('Checking if assignment is answerable or not...')
        time.sleep(5)
        while self.page.locator('[data-testid="stop-button"]').is_visible():
            time.sleep(2)
        resp = self.page.locator('div.text-base').all()[-3].inner_text()
        resp = resp.replace("ChatGPT\nMemory updated\n\n", "").replace("ChatGPT\n\n", "").replace("\n\n4o mini", "").replace("\n\n4o","").replace("\nIs this conversation helpful so far?", "").replace("ChatGPT\njson\nCopy code\n", "").replace('Which response do you prefer?\nYour choice will help make ChatGPT better.\nChatGPT\nResponse 1', "").replace('ChatGPT\nResponse 2', '')
        self.gpt_check_asg_url = self.page.url
        self.page.context.storage_state(path=self.gpt_state_conf)
        try:
            return json.loads(resp)
        except Exception as e:
            print(f'[bold red]Something went wrong While validating assignment: [/]{e}')
            print("By default sending question to ChatGPT")
            return {"send_to_chatbot":True, "reason":""}




"""
This file is a part of Internshala-bot Package. 
Github - https://github.com/Eviltr0N/internshala-bot
Written by - Mayank Lodhi
"""


import os
from rich import print

config_file = 'resume.ini'


if not os.path.exists(config_file):
    with open(config_file, 'w', encoding='latin-1') as file:
        file.write('\n#Enter your Skills Here \n\n')
        file.write('SKILLS = """\n\n\nPython, sql, PowerBi\n Azure, AWS, Google Cloud, Colorful Rain Cloud\nTime Management, Excellent Communication Skills etc.\n\n\n"""')

        file.write('\n\n#Enter Your Certificates and previous Internship Experiences here\n\n')
        file.write('CERTIFICATES = """\n\n\n1. Python basics by Microsoft\n2. Ai Essentials by Nvidia\n3. 6 Months AI & ML internship at Eviltron. etc\n\n\n"""')

        file.write('\n\n#Add your personal/professional projects here, enter name and describe it if you want.\n #You can you ChatGpt to summarize your project in points.\n\n')
        file.write('PROJECTS = """\n\n\n1. AI Chatbot\n* Created an AI chatbot using OpenAi api.\n*Designed an interactive UI for it using React.\n*Added support to talk with aliens form other galaxies.\n*Added inter galaxy translation.\n\n2. Face Recognition Bot \n* Created an face Recognition bot using Rust and Assembly. \n*Added advanced face Recognition capabilities to identify ghosts and aliens.\n* Added age detection feature to detect age of aliens and ghosts.\n\n\n"""')

        print('\nPlease Open [bold green]"resume.ini" [/]file using any text editor and add your [bold yellow]Skills, Certificates and Projects.[/] After adding those restart the script.\n')
        exit()


with open(config_file, 'r', encoding='latin-1', errors='ignore') as file:
    config_data = file.read().replace(";", "#")

try:
    exec(config_data)
except (IndentationError,SyntaxError) as e:
    print('[blod red]There is some indentation error with resume.ini file make sure you follow proper python indentation in resume.ini file.\n\nIt contains three multiline string variables. EDIT YOUR RESUME.INI FILE and RUN again. If you think you messed up everything then you can deleate the "resume.ini file and RUN again to recreate it then edit it."[/]')
    exit()

try:
    Skills = SKILLS
except:
    print("[bold red]Your skills dosent found in resume.ini file\n If you want to continue without skills press Enter else add skills and restart script.\n\n[/]")
    _ = input("Press Enter: ")
    Skills = '\nNO skills specified assume skills accordingly'

try:
    Certifications = CERTIFICATES
except:
    print("[bold red]Your Certifications dosent found in resume.ini file\n If you want to continue without Certifications press Enter else add skills and restart script.\n\n[/]")
    _ = input("Press Enter: ")
    Certifications = 'No certificates assume accordingly'


try:
    Projects = PROJECTS
except:
    print("[bold red]Your projects dosent found in resume.ini file\n If you want to continue without projects press Enter else add skills and restart script.\n\n[/]")
    _ = input("Press Enter: ")
    Projects = "No projects specified assume accordingly"


#======================================================================

def cover_letter_prompt_format(profile, company, about, skills):
    prompt=f'''
I want you to write a cover letter for an Remote Internship, \
I will provide you The internship description, their skill requirements. \
I will also give you my skills, my certifications, my projects. \
You have to understand the internship description and my whole portfolio then mention \
key things from my portfolio that are relavent to the internship description. \
Keep cover letter simple but effective. \
Use general Indian english so it looks like its written by human. \
Below My skills, certifications, projects and internship description, their skill requirements are attached. \
Dont use email formatting as I am not sending this in an email. \
Also Don't include header(such as Dear Hiring Manager... etc) and footer(such as Sincerly...etc). \
If my skill set lacks any of their skill requirement then assume that I possess that skill too. \


My Skills
```
{Skills}
```
Certifications
```
{Certifications}
```
My Projects
```
{Projects}
```

Internship Description
```
{profile}

{about}

Skills Required
{skills}
```
Company Name: {company}

Important - The letter must be less than 2000 characters.
'''
    return prompt

#====================================================================

def assignment_prompt_format(profile, company, about, skills, question):
    prompt=f'''
I am applying for an internship whose description is attached below and their skills requirement too. \
They are asking that `{question}` \
I have attached my skills, certifications, projects also. \
I want you to understand their requirements and their question and answer it accordingly while keeping in mind my portfolio. \
If they are asking about any working shift or relocation to other place then i am totally confortable in all situations. \
If they are asking that about scheduled classes, assignments, or exams in the next 6 months then I dont have any of those as I am recently graduated and I can devote my time fully on internship. \
Use general Indian english so it looks like its written by human & don't exhagurate your words. \
If my skill set lacks any of their skill requirement then assume that I possess that skill too. \
Keep Your answer to the point, short and precise.

My Skills
```
{Skills}
```
Certifications
```
{Certifications}
```
My Projects
```
{Projects}
```

Internship Description
```
{profile}

{about}

Skills Required
{skills}
```
Company Name: {company}

Important - In your response just give me answer following previous instructions and do not \
include question or any other text in you response besides answer of the question.
'''

    return prompt


def assignment_validation_prompt(profile, question):
    header='''

I am making a project which answers Internship Assignment Questions while applying for Internship Using an AI chatbot. The Ai chatbot has given My Personal portifolio which includes My Skills. Projects and Certification. It also given My working shifts and If I can relocate or not. I dont want that Ai chatbot will answer questions related to Personal things or related to information which is not inside the portfolio which is given to the Chatbot.You have to act as Moderator that I will give you question which will then passed to Ai chatbot, You have to check if the Question is related to :- 
`
Any personal information such as asking Grades/Percentages/CGPA.
If Question Contains any link/Url.
Asking question related Salary/Stipend.
Asking for LinkedIn/Github Profile Links.
Asking for Personal Achivements which are not related to Internshpi Profile.

`
If the question is related to above mention things then It should not answered by Ai chatbot and You have to Give Your Response In JSON Format. Here is Your Response Format - 
`
{
"send_to_chatbot": true / false,
"reason": Reason which thing is asked
}
`
Here are few Examples - 
Example - 1
"""
Profile - Data Analytics
Question - Candidates who have completed graduation or are in their last year of graduation (2024) would be considered for the internship. Please state your year of passing. Also, mention if you scored more than 70% in 10th, 12th, and graduation.
"""
Your Response Should be - 
""
{"send_to_chatbot": false, "reason":"Asking For Educational Details"}
""
Example - 2
"""
Profile - Machine Learning
Question - Provide us your Linkedin and Github.
"""
Your Response Should be - 
""
{"send_to_chatbot": false, "reason":"Asking For Linkedin/Github"}
""
Example - 3

"""
Profile - Machine Learning
Question - What will be the answer of this question https://docs.google.com/document/yup/sheisgorgeous.
"""
Your Response Should be - 
""
{"send_to_chatbot": false, "reason":"Contains Link"}
""
Example - 3

"""
Profile - Ai Engineer
Question - How do you ensure that you are aware of the latest trends, breakthroughs, and news in the field of artificial intelligence?.
"""
Your Response Should be - 
""
{"send_to_chatbot": true, "reason":""}
""

Example - 4

"""
Profile - Backend Engineer
Question - Have you posses any preoir experience in the same field? If yes please mention.
"""
Your Response Should be - 
""
{"send_to_chatbot": true, "reason":""}
""
'''

    footer = f'''
Now here is the question which you have to check - 
```
Profile - {profile}
Question -  {question}

```
Keep in mind you should only give response in desired format.
'''

    return header + footer



"""
This file is a part of Internshala-bot Package. 
Github - https://github.com/Eviltr0N/internshala-bot
Written by - Mayank Lodhi
"""

import pandas as pd
import time
import os
from rich import print


class df_success:
    def __init__(self):
        columns = ["Sr. No", "Profile", "Company", "Applied On", "Skills", "Status", "link"]
        self.df = pd.DataFrame(columns=columns)
        self.sr_no = 0

    def add(self, profile, company, skills, status, link):
        self.sr_no += 1
        new_row = pd.DataFrame({
            "Sr. No": [self.sr_no],
            "Profile": [profile],
            "Company": [company],
            "Applied On": [time.strftime("%d-%B-%Y",time.localtime())],
            "Skills": [skills.replace("\n", "<br>")],
            "Status": [status],
            "link": [link]
        })
        self.df = pd.concat([self.df, new_row], ignore_index=True)
        
    def generate(self):
        self.df['Profile'] = self.df.apply(lambda row: f'<a href="{row["link"]}">{row["Profile"]}</a>', axis=1)
        html_table = self.df.drop(columns=['link']).to_html(index=False, escape=False)
        html_template = f"""
        <!DOCTYPE html>
        <html lang="en">
        <head>
            <meta charset="UTF-8">
            <meta name="viewport" content="width=device-width, initial-scale=1.0">
            <title>Success Report</title>
            <style>
                body {{
                    font-family: Arial, sans-serif;
                    margin: 20px;
                }}
                table {{
                    width: 100%;
                    border-collapse: collapse;
                    margin-bottom: 20px;
                }}
                table, th, td {{
                    border: thin solid #e0e0e0;
                }}
                th, td {{
                    padding: 12px;
                    text-align: left;
                }}
                th {{
                    background-color: #3b3041ed;
                    color: white;
                }}
                tr:nth-child(even) {{
                    background-color: #f2f2f2;
                }}
            </style>
        </head>
        <body>
            <h1>Successfull Application Report </h1> <h3><a href="https://github.com/Eviltr0N/internshala-bot">by Internshala_bot</a></h3>
            {html_table}
        </body>
        </html>
        """
        report_dir = os.path.join(os.getcwd() , 'reports')
        if not os.path.exists(report_dir):
            os.makedirs(report_dir)
        current_time = time.strftime("%H_%M_%d_%B")
        filename = f"SUCCESS_{current_time}.html"

        with open(os.path.join(report_dir, filename), 'w') as file:
            file.write(html_template)

        print("[bold green]Success Report Saved in: [/]", os.path.join(report_dir, filename))


class df_failed():
    def __init__(self):
        columns = ["Sr. No", "Profile", "Company", "Skills", "Reason", "Cover Letter", "link"]
        self.df = pd.DataFrame(columns=columns)
        self.sr_no = 0

    def add(self, profile, company, skills, reason, cover_letter, link):
        self.sr_no += 1
        new_row = pd.DataFrame({
            "Sr. No": [self.sr_no],
            "Profile": [profile],
            "Company": [company],
            "Skills": [skills.replace("\n", "<br>")],
            "Reason": [reason],
            "Cover Letter": [cover_letter.replace("\n", "<br>")],
            "link": [link]
        })
        self.df = pd.concat([self.df, new_row], ignore_index=True)

    def create_expandable_cover_letter(self,text):
        short_text = text[:100] + "..."
        return f'''
        <div class="cover-letter">
            <span class="short-text">{short_text}</span>
            <span class="full-text" style="display:none;">{text}</span>
            <button onclick="toggleCoverLetter(this)">Read More</button>
        </div>
        '''

        
    def generate(self):
        self.df['Profile'] = self.df.apply(lambda row: f'<a href="{row["link"]}">{row["Profile"]}</a>', axis=1)
        df_html = self.df.drop(columns=['link']).copy()
        df_html['Cover Letter'] = df_html['Cover Letter'].apply(self.create_expandable_cover_letter)
        html_table = df_html.to_html(index=False, escape=False)

        html_template = f"""
            <!DOCTYPE html>
            <html lang="en">
            <head>
                <meta charset="UTF-8">
                <meta name="viewport" content="width=device-width, initial-scale=1.0">
                <title>Failed to Apply Report</title>
                <style>
                    body {{
                        font-family: Arial, sans-serif;
                        margin: 20px;
                    }}
                    table {{
                        width: 100%;
                        border-collapse: collapse;
                        margin-bottom: 20px;
                    }}
                    table, th, td {{
                        border: thin solid #e0e0e0;
                    }}
                    th, td {{
                        padding: 12px;
                        text-align: left;
                    }}
                    th {{
                        background-color: #3b3041ed;
                        color: white;
                    }}
                    tr:nth-child(even) {{
                        background-color: #f2f2f2;
                    }}
                    .cover-letter {{
                        position: relative;
                    }}
                    .cover-letter button {{
                        position: absolute;
                        bottom: 0;
                        right: 0;
                        background: none;
                        border: none;
                        color: blue;
                        cursor: pointer;
                        text-decoration: underline;
                    }}
                </style>
                <script>
                    function toggleCoverLetter(button) {{
                        var fullText = button.previousElementSibling;
                        var shortText = fullText.previousElementSibling;
                        if (fullText.style.display === "none") {{
                            fullText.style.display = "inline";
                            shortText.style.display = "none";
                            button.textContent = "Read Less";
                        }} else {{
                            fullText.style.display = "none";
                            shortText.style.display = "inline";
                            button.textContent = "Read More";
                        }}
                    }}
                </script>
            </head>
            <body>
               <h1>Failed to Apply Report </h1> <h3><a href="https://github.com/Eviltr0N/internshala-bot"> by Internshala_bot </a></h3>
                {html_table}
            </body>
            </html>

            """
        report_dir = os.path.join(os.getcwd(), "reports")
        if not os.path.exists(report_dir):
            os.makedirs(report_dir)
        current_time = time.strftime("%H_%M_%d_%B")
        filename = f"FAILED_{current_time}.html"

        with open(os.path.join(report_dir, filename), 'w') as file:
            file.write(html_template)

        print("[bold red]Failed Report Saved in: [/]", os.path.join(report_dir, filename))




from setuptools import setup, find_packages

with open('README.md') as f:
    readme=f.read()


setup(
    name="internshala-bot",
    version="0.0.2",
    author="Mayank Lodhi",
    author_email="",
    description="Package to automate internship application process on Internshala using ChatGPT.",
    long_description=readme,
    long_description_content_type="text/markdown",
    license="Apache-2.0",
    url="https://github.com/Eviltr0N/internshala-bot",
    project_urls={
        'Bug Reports': 'https://github.com/Eviltr0N/internshala-bot/issues',
        'Source Code': 'https://github.com/Eviltr0N/internshala-bot/',
    },
    packages=find_packages(),
    install_requires=[
        "undetected-playwright-patch==1.40.0.post1700587210000",
        "rich==13.7.1",
        "argparse",
        "requests",
        "requests-html",
        "pandas",
        "lxml_html_clean",

    ],
    classifiers=[
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
    # python_requires=">=3.8",
    entry_points={
        "console_scripts": [
            "internshala_bot=internshala_bot.main:main",
        ],
    },
)


"""
This file is a part of Internshala-bot Package. 
Github - https://github.com/Eviltr0N/internshala-bot
Written by - Mayank Lodhi
"""

from undetected_playwright.sync_api import sync_playwright
import os
import time
import argparse
import requests
from requests_html import HTML
from rich.progress import track
from rich import print
from rich.theme import Theme
from .chat_gpt import chat
from .generate_report import df_failed, df_success



class internshala:
    def __init__(self, browser):
        self.profile = None
        self.company = None
        self.skills = None
        self.about = None
        self.internship_id = None
        self.int_browser = None
        self.gpt_browser = None
        self.browser = browser
        self.scn_size = {"width":800, "height":800}

        config_dir = os.path.join(os.getcwd(), '.config')
        if not os.path.exists(config_dir):
            os.makedirs(config_dir)
        self.intern_state_conf = os.path.join(config_dir, 'internshala_state.json')
        self.gpt_state_conf = os.path.join(config_dir, 'chat_gpt_state.json')

        if not os.path.exists(self.intern_state_conf):
            print("[bold yellow]Please log-in to your Internshala Account[/]\n")
            self.login_internshala()
        else:
            self.int_browser = self.browser.new_context(storage_state=self.intern_state_conf,
            viewport=self.scn_size,
            )

        if not os.path.exists(self.gpt_state_conf):
            print('[bold yellow]Please log-in to your ChatGPT Account[/]\n')
            self.login_chat_gpt()
        else:
            current_time = time.time()
            creation_time = os.path.getctime(self.gpt_state_conf)
            file_age_hours = (current_time - creation_time) / 3600
            if file_age_hours > 3:
                os.remove(self.gpt_state_conf)
                print('[bold yellow]Please log-in to your ChatGPT Account[/]\n')
                self.login_chat_gpt()
            else:
                self.gpt_browser = self.browser.new_context(storage_state=self.gpt_state_conf,
                viewport=self.scn_size,
                )
                

    def login_internshala(self):
        page = self.browser.new_page()
        page.goto('https://internshala.com/login/user', timeout=60000, wait_until='networkidle')
        dash_url = 'https://internshala.com/student/dashboard'
        start_time = time.time()
        while page.url != dash_url and not page.locator('div.profile_icon_right').is_visible():
            time.sleep(1)
            if time.time() - start_time > 240:
                print("[bold red]Timeout... Please Restart The script to login again.[/]\n")
                exit()

        storage = page.context.storage_state(path=self.intern_state_conf)
        self.int_browser = self.browser.new_context(storage_state=self.intern_state_conf)
        print('[bold green]Internshala Login Successful Closing Browser in 5 Seconds...[/]\n')
        time.sleep(4)
        page.close()
        
    def login_chat_gpt(self):
        page = self.browser.new_page()
        page.goto('https://chat.openai.com' , timeout=60000, wait_until='networkidle')
        time.sleep(1)
        try:
            page.locator('[data-testid="login-button"]').click()
        except:
            print("[bold yellow]Click on Login button and Login using Your Emali and Password...\n[/]")
        start_time = time.time()
        while not page.locator('[data-testid="profile-button"]').is_visible():
            time.sleep(1)
            if time.time() - start_time > 180:
                print("[bold red]Timeout... Please Restart The script to login again.[/]\n")
                exit()

        storage = page.context.storage_state(path=self.gpt_state_conf)
        self.gpt_browser = self.browser.new_context(storage_state=self.gpt_state_conf)
        print('[bold green]ChatGPT Login Successful Closing Browser in 5 Seconds...[/]\n')
        time.sleep(4)
        page.close()

    def get_internship_info(self, url):
        self.intshp_url = url
        self.page = self.int_browser.new_page()
        self.page.goto(url, timeout=60000, wait_until='networkidle')
        if self.page.get_by_text("Custom job").is_visible():
            self.page.locator('//*[@id="close_popup"]').click()

        self.internship_id = self.page.locator('div[id^="individual_internship"]').first.get_attribute('internshipid')
        self.profile = self.page.locator(f'#individual_internship_{self.internship_id} > div.internship_meta > div.individual_internship_header > div.company > div.heading_4_5.profile').inner_text()
        self.company = self.page.locator(f'#individual_internship_{self.internship_id} > div.internship_meta > div.individual_internship_header > div.company > div.heading_6.company_name > div > a').inner_text()
        self.about = self.page.locator(f'#details_container > div.detail_view > div.internship_details > div:nth-child(2)').inner_text()
        round_tabs_count = self.page.locator('.round_tabs_container').count()
        skills_loc = self.page.locator('.round_tabs_container').first
        if skills_loc.is_visible() and round_tabs_count >= 2:
            skills_lis = skills_loc.locator('.round_tabs').all_inner_texts()
            self.skills = ""
            for x in skills_lis:
                self.skills+= f"{x}\n"
        else:
            self.skills = "`Company dosent mentioned perticular skills, assume skills according to Internship Description`"
        if self.page.get_by_role("button", name="Already Applied").is_visible():
            print("[bold yellow]Already applied: [/]", self.page.url)
            self.page.close()
            return True
        else:
            apply_button = self.page.get_by_role("button", name="Apply now")
            apply_button.click()
            return False
        self.page.context.storage_state(path=self.intern_state_conf)
    
    def fill_app_form(self, GPT, success, failed, validate_assignment_question):
        if self.page.url == 'https://internshala.com/student/resume?detail_source=resume_intermediate':
            self.page.locator('#layout_table > div.proceed-btn-container > button').click()

        cover_letter = self.page.locator('//*[@id="cover_letter_holder"]/div[1]')
        self.cover = GPT.get_cover_letter(self.profile, self.company, self.about, self.skills)
        cover_letter.fill(self.cover)
        assignments = self.page.locator('.form-group.additional_question')
        count = assignments.count()
        print('Total assignments: ', f'[bold green]{count}[/]\n')
        self.que_text = None
        for x in range(count):
            self.que_text = assignments.nth(x).locator("div.assessment_question > label").inner_text()
            if validate_assignment_question:
                res = GPT.assmnt_is_valid(self.profile, self.que_text)
                if res["send_to_chatbot"]:        
                    self.answer = GPT.get_assignment_answer(self.profile, self.company, self.about, self.skills, self.que_text)
                    print("[green]Assignment Sent to ChatGPT...[/]\n",)
                else:
                    failed.add(self.profile, self.company, self.skills, res["reason"], self.cover, self.intshp_url)
                    print("[bold red]YOU HAVE TO MANUALLY APPLY TO THIS INTERNSHIP[/]\n" )
                    print("[bold yellow]Reason: [/]", f'{res["reason"]}\n')
                    self.page.close()
                    return False
            else:
                print('[green]No validation Assignment directly sent to GPT[/]')
                self.answer = GPT.get_assignment_answer(self.profile, self.company, self.about, self.skills, self.que_text)
            
            ans_loc = assignments.nth(x).locator('textarea[id^="text_"]')
            ans_loc.fill(self.answer)
        submit_loc = self.page.locator('//*[@id="submit"]')
        success.add(self.profile, self.company, self.skills, "Applied", self.intshp_url)
        submit_loc.click()
        time.sleep(1)
        self.page.context.storage_state(path=self.intern_state_conf)
        self.page.close()
 
    def update_resume_skills(self):
        cookies = self.page.context.cookies()
        for x in cookies:
            if x["name"]=="csrf_cookie_name":
                csrf = x
        
        cookie_names = ['AWSALBCORS', 'l', 'lc', 'lv', 'PHPSESSID', "csrf_cookie_name"]
        req_cookies = {cookie['name']: cookie['value'] for cookie in cookies if cookie['name'] in cookie_names}
        
        headers = {

            'Sec-Ch-Ua': '"Not(A:Brand";v="24", "Chromium";v="122"',
            'X-Requested-With': 'XMLHttpRequest',
            'Content-Type': 'application/x-www-form-urlencoded; charset=UTF-8',
            'Sec-Ch-Ua-Mobile': '?0',
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/122.0.6261.112 Safari/537.36',
            'Sec-Ch-Ua-Platform': '"Windows"',
            'Origin': 'https://internshala.com',
            'Sec-Fetch-Site': 'same-origin',
            'Sec-Fetch-Mode': 'cors',
            'Sec-Fetch-Dest': 'empty',
            'Referer': 'https://internshala.com/student/resume?detail_source=resume_direct',
            'Accept-Language': 'en-US,en;q=0.9',
            'Accept-Encoding': 'gzip, deflate, br',
            'Priority': 'u=1, i'
            }

        url = 'https://internshala.com/student/skills_submit'
        skill_list = self.skills.split("\n")
        _ = skill_list.pop(-1)
        print('[bold green]Adding Required Skills into Intternshala Resume...[/]\n')
        for skill_name in skill_list:
            data = {
                    'skill_name': skill_name,  
                    'add_skill_from': 'student_profile_form',
                    'skill_level': 'Intermediate',
                    'csrf_test_name': req_cookies["csrf_cookie_name"]
                }
            res = requests.post(url, headers=headers, cookies=req_cookies, data=data)
            if res.json()["success"] == False:
                print(skill_name, f'[red]{res.json()["errorThrown"]}[/]' )
            else:
                print(skill_name, "[green]Added Successfully[green]")
            req_cookies = res.cookies

        csrf.update({"value": req_cookies["csrf_cookie_name"]})
        self.page.context.add_cookies([csrf])

    @staticmethod
    def check_hiring_stats(url):
        res = requests.get(url)
        html = HTML(html=res.text)
        try:
            opp = int(html.find("#details_container > div.detail_view > div.internship_details > div.activity_section > div.activity_container > div:nth-child(2) > div")[0].text.split(" ")[0])
        except IndexError as e:
            print(html.xpath("/html/body/div[1]/div[20]/div/p")[0].text)
            opp = 0
        try:
            cand = int(html.find("#details_container > div.detail_view > div.internship_details > div.activity_section > div.activity_container > div:nth-child(3) > div")[0].text.split(" ")[0])
        except IndexError as e:
            cand = 0
        if opp > 10 and cand <= 1:
            should_apply = False
        else:
            should_apply = True
        return {"posted": opp, "hired":cand, "should_apply": should_apply}

    @staticmethod
    def get_interns_list(url):
        res= requests.get(url)
        html = HTML(html=res.text)
        curr_page = int(html.find('#pageNumber', first=True).text)
        total_page =  int(html.find('#total_pages', first=True).text)
        isp_list=[]
        for x in range(curr_page, total_page+1):
            new_url = url+f"page-{x}/"
            res= requests.get(new_url)
            html = HTML(html=res.text)
            data_hrefs = [element.attrs.get('data-href', '') for element in html.find('div.individual_internship') if 'data-href' in element.attrs]
            profiles = [x.text for x in html.find(".job-internship-name") ]
            for ind, x in enumerate(profiles):
                isp_list.append({
                                "link": f"https://internshala.com{data_hrefs[ind]}",
                                "profile": x
                                })
        return isp_list

    @staticmethod
    def get_final_links(filter_page_url, additional_filters=None):
        all_internships = internshala.get_interns_list(filter_page_url)
        filter_from_url = filter_page_url.split("/")[4].replace("job", "internship").split("-internship")[0].replace("work-from-home-", "").replace("part-time-", "").replace(",", "-").split("-")
        internship_filters = filter_from_url
        if additional_filters != None:
            for x in additional_filters:
                for c in x.lower().split(" "):
                    internship_filters.append(c)
        print("Filtering Internships for: ", internship_filters)
        filtered_urls = []
        for x in track(all_internships, description="Fetching Internships"):
            for elem in x["profile"].lower().split(" "):
                if elem in internship_filters and x["link"] not in filtered_urls:
                    res = internshala.check_hiring_stats(x["link"])
                    if res["should_apply"]:
                        filtered_urls.append(x["link"])
        return filtered_urls


def main():

    parser = argparse.ArgumentParser(description='A program to automate Internship Application process on Internshala using ChatGPT. \n\nSee full Guide on Github: https://github.com/Eviltr0N/internshala-bot/')
    parser.add_argument('url', nargs='*', help='Enter url of search page of Internshala After applying desired filters or url of Single Internship detail page')
    parser.add_argument('-f', '--filters', '-filters', '--filter', nargs='*', default=[],dest='filters', metavar='Internship_Roles' , help='You can specify additional role filter for Internship which are not available in internshala filter Such as HTML developer, Linux Specilist etc')
    parser.add_argument('--skip_assignment_validation', '--skip', dest='skip',action='store_false', help="This flag is used to disable assignment validation check which is used to check if any personal information is asked in Assignment Question.")

    arguments = parser.parse_args()

    validate_assignment_question = arguments.skip #Must be true to validate assignment question
    additional_filters = arguments.filters
    urls = arguments.url
    links=[]
    if len(urls) == 0:
        print("\nPlease Enter a url of search page of Internshala After applying desired filters or url of Single Internship detail page \n\nYou can either enter url of search page of Internshala After applying desired filters \n[bold yellow]Example[/]: https://internshala.com/internships/work-from-home/ \n\n     OR     \n\nYou can enter single/multiple urls of Individual Internship detail page seperated by spaces \n[bold yellow]Example[/]: \nFor Individual Internship: \nhttps://internshala.com/internship/detail/x-y-z \n\nFor Multiple Individual Internships: \nhttps://internshala.com/internship/detail/a-b-c https://internshala.com/internship/detail/x-y-z https://internshala.com/internship/detail/s-h-e \n\nFull Guide on Github - https://github.com/Eviltr0N/internshala-bot?tab=readme-ov-file#how-to-use")
        exit()
    if len(urls) > 1:
        for url in urls:
            if url.find("internshala.com/internship/detail/") != -1:
                links.append(url)
            elif url.find("internshala.com/internships/") != -1:
                print("[bold red]Please Don't Combine[/] Single Internship url [bold red]with[/] Multiple Internship Search page url. \n\nYou can either enter url of search page of Internshala After applying desired filters \n[bold yellow]Example[/]: https://internshala.com/internships/work-from-home/ \n\n     OR     \n\nYou can enter single/multiple urls of Individual Internship detail page seperated by spaces \n[bold yellow]Example[/]: \nFor Individual Internship: \nhttps://internshala.com/internship/detail/x-y-z \n\nFor Multiple Individual Internships: \nhttps://internshala.com/internship/detail/a-b-c https://internshala.com/internship/detail/x-y-z https://internshala.com/internship/detail/s-h-e \n\nFull Guide on Github - https://github.com/Eviltr0N/internshala-bot?tab=readme-ov-file#how-to-use")
                exit()
            else:
                print("[bold red]Invalid url: [/]", url)
    else:
        if urls[0].find("internshala.com/internship/detail") != -1:
            links.append(urls[0])
        elif urls[0].find("internshala.com/internships/") != -1:
            url=urls[0]
        else:
            print("[bold red]Please Enter a valid url of Internshala's Search page or Internship Page such as [/]https://internshala.com/internships/work-from-home/, or https://internshala.com/internship/detail/ \n\nFull Guide on Github - https://github.com/Eviltr0N/internshala-bot?tab=readme-ov-file#how-to-use")
            exit()
    
    success = df_success()
    failed = df_failed()
    
    with sync_playwright() as p:
        print('Starting...')
        args = []
        args.append("--disable-blink-features=AutomationControlled")
        browser = p.chromium.launch(args=args, headless=False)  # Run in headful mode
        
        intern = internshala(browser)
        GPT = chat(intern.gpt_browser)

        if len(links) == 0:
            links = intern.get_final_links(url, additional_filters)

        for link in track(links, description="Processing"):
            print(f"[bold green]Applying at: [/]{link}")
            intern = internshala(browser)
            already_applied = intern.get_internship_info(link)
            if not already_applied:
                intern.update_resume_skills()
                intern.fill_app_form(GPT, success,failed, validate_assignment_question)
            else: 
                print("[bold yellow]Already Applied. Skipping...[/]")
        
        success.generate()
        failed.generate()



if __name__ == "__main__":
    main()


"""
This file is a part of Internshala-bot Package. 
Github - https://github.com/Eviltr0N/internshala-bot
Written by - Mayank Lodhi
"""

from .main import main

if __name__ == "__main__":
    main()

